<?php
/*
Plugin Name: UtopiaPay for Easy Digital Downloads
Plugin URI: https://utopiapay.io
Description: Accept payments in Crypton (CRP) and Utopia USD (UUSD) via UtopiaPay.
Version: 1.0.14
Author: UtopiaPay
Author URI: https://utopiapay.io
License: GPLv2 or later
Text Domain: utopiapay
*/

/**
 * UtopiaPay.io Gateway
 * 
 *
 * Provides a UtopiaPay Payment Gateway for Easy Digital Downloads
 * @author 		UtopiaPay gateway plugin by Teo2Peer & Saahil
 */



// check if woocommerce is active and throw and error that Utopiapay for EDD is not compatible with WooCommerce
if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
    
    function utopiapay_edd_incompatibility_notice() {
        ?>
<div class="error">
    <p><?php _e( 'Utopiapay for Easy Digital Downloads is not compatible with WooCommerce. Please deactivate WooCommerce and try again.', 'utopiapay' ); ?>
    </p>
</div>
<?php
    }
    add_action( 'admin_notices', 'utopiapay_edd_incompatibility_notice' );
    
    return;
}


// check if EDD is active, else stop the plugin
if ( !in_array( 'easy-digital-downloads/easy-digital-downloads.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
    return;
}


// registers the gateway
function register_gateway($gateways) {
        $gateways['UtopiaPay']= array(
            'admin_label' => __('UtopiaPay', 'easy-digital-downloads'),
            'checkout_label' => __('UtopiaPay',
                'easy-digital-downloads')
        );
    

    return $gateways;
}
add_filter('edd_payment_gateways', 'register_gateway');



// remove cc form
function edd_UtopiaPay_remove_cc_form() {
    return false;
}
add_filter('edd_require_billing_address', 'edd_UtopiaPay_remove_cc_form');

function pw_edd_UtopiaPay_cc_form() {
	// register the action to remove default CC form
	return;
}
add_action('edd_UtopiaPay_cc_form', 'pw_edd_UtopiaPay_cc_form');
 


function jp_disable_billing_details() {
	remove_action( 'edd_after_cc_fields', 'edd_default_cc_address_fields' );
    
}
add_action( 'init', 'jp_disable_billing_details' );



function register_payment_icon($payment_icons)
    {
        $payment_icons[plugins_url('utopiapay.png', __FILE__)] = "UtopiaPay";

        return $payment_icons;
    }
add_filter('edd_accepted_payment_icons','register_payment_icon', 10, 1);



///Register the payment gateways setting section
function register_gateway_section($gateway_sections)
{
    $gateway_sections['utopiapay'] = __('UtopiaPay', 'easy-digital-downloads');
    

    return $gateway_sections;
}



if (is_admin()) {
    add_filter('edd_settings_sections_gateways','register_gateway_section');
    add_filter('edd_settings_gateways','register_gateway_settings');
}


// add CRP and uUSD as accepted currencies
function register_currencies($currencies)
{
    $currencies['CRP'] = __('Crypton', 'easy-digital-downloads');
    $currencies['uUSD'] = __('Utopia USD', 'easy-digital-downloads');

    return $currencies;
}
add_filter('edd_currencies', 'register_currencies');

// set the symbol
function register_currency_symbol($currency_symbol, $currency)
{
    switch ($currency) {
        case 'CRP':
            $currency_symbol = 'CRP';
            break;
        case 'uUSD':
            $currency_symbol = 'uUSD';
            break;
    }

    return $currency_symbol;
}



// adds the settings to the Payment Gateways section
function register_gateway_settings($gateway_settings)
{
    $default_utopiapay_settings = array(
        'utopiapay' => array(
            'id' => 'utopiapay',
            'name' => '<strong>' . __('UtopiaPay Payment Gateway Settings', 'easy-digital-downloads') . '</strong>',
            'type' => 'header',
        ),
        // description
        // 'utopiapay_description' => array(
        //     'id' => 'utopiapay_description',
        //     'name' => __('Description', 'easy-digital-downloads'),
        //     'desc' => __('This controls the description which the user sees during checkout.', 'easy-digital-downloads'),
        //     'type' => 'text',
        //     'std' => __('Pay in Crypton (CRP) and Utopia USD (UUSD) via UtopiaPay', 'easy-digital-downloads'),
        //     'size' => 'regular',
        //     'default' =>  __('Pay in Crypton (CRP) and Utopia USD (UUSD) via UtopiaPay', 'easy-digital-downloads'),
        // ),
        // merchant id type number
        'utopiapay_merchant_id' => array(
            'id' => 'utopiapay_merchant_id',
            'name' => __('UtopiaPay ID', 'easy-digital-downloads'),
            'desc' => __('Enter your UtopiaPay ID', 'easy-digital-downloads'),
            'type' => 'text',
            'std' => '',
            'size' => 'regular'
        ),
        // token
        'utopiapay_token' => array(
            'id' => 'utopiapay_token',
            'name' => __('UtopiaPay Token', 'easy-digital-downloads'),
            'desc' => __('Enter your UtopiaPay Token', 'easy-digital-downloads'),
            'type' => 'text',
            'std' => '',
            'size' => 'regular'
        ),

        // inside outside utopia
        'utopiapay_inside_outside_utopia' => array(
            'id' => 'utopiapay_inside_outside_utopia',
            'name' => __('Inside or Outside Utopia', 'easy-digital-downloads'),
            'desc' => __('Select if your site inside or outside Utopia', 'easy-digital-downloads'),
            'type' => 'select',
            'options' => array(
                'inside' => __('Inside Utopia', 'easy-digital-downloads'),
                'outside' => __('Outside Utopia', 'easy-digital-downloads'),
                'auto' => __('Auto-detect (not secure and can fail)', 'easy-digital-downloads'),
            ),
            'std' => 'inside'
        ),
        // public.pem
        'utopiapay_secret_key' => array(
            'id' => 'utopiapay_secret_key',
            'name' => __('UtopiaPay secret Key', 'easy-digital-downloads'),
            'desc' => __('Enter your UtopiaPay Secret Key', 'easy-digital-downloads'),
            'type' => 'text',
            'std' => '',
            'size' => 'large',

        ),

    );

    $default_utopiapay_settings = apply_filters('edd_default_utopiapay_settings', $default_utopiapay_settings);
    $gateway_settings['utopiapay'] = $default_utopiapay_settings;

    return $gateway_settings;
}


// processes the payment
function pw_edd_process_payment($purchase_data) {
    global $edd_options;

    $order = edd_insert_payment($purchase_data);

    try {
        // get the coinfig data
        $token = edd_get_option('utopiapay_token');
        $id = edd_get_option('utopiapay_merchant_id');
        $inside_outside_utopia = edd_get_option('utopiapay_inside_outside_utopia');
        $secretKey = edd_get_option('utopiapay_secret_key');
        $order_id = $order;
        $success_url = edd_get_success_page_uri();
        $cancel_url = edd_get_failed_transaction_uri();
        $currency = edd_get_currency();
        $ipn_url = trailingslashit(home_url()) . '?edd-listener=utopiapay';
        $amount = $purchase_data['price'];
        $url = get_site_url();




        // WordPress get url 

        // check if the order is inside or outside utopia
        if ($inside_outside_utopia == 'inside') {
            $isInside = "true";
            $uUrl = "http://utopiapay";
        } else if ($inside_outside_utopia == 'outside') {
            $isInside = "false";
            $uUrl = "https://utopiapay.io";
        }else if ($inside_outside_utopia == 'auto') {
            // check if the host url from server host is http or https (inside or outside)
            // we use $_SERVER['HTTPS'] instead of $url due user can host the same site 
            // with same url inside utopia
            if(isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on'){   
                $isInside = "false";
                $uUrl = "https://utopiapay.io";  
            }else  {
                $isInside = "true";
                $uUrl = "http://utopiapay"; 
            } 
        }

		

		
        // generate the data to send to UtopiaPay Gateway
        $data = array(
            "amount" => $amount,
            "orderID" => $order_id,
            "token" => $token,
            "webTo" => $success_url,
            "webCancel" => $cancel_url,
            "siteUrl" => $url,
            "ipnUrl" => $ipn_url,
            "isInside" => $isInside,
            "id" => $id,
            "currency" => $currency,
        );
        $json = json_encode($data,JSON_FORCE_OBJECT,1024);

        $dataToCrypt = $json;
        $dataToCrypt = base64_encode($dataToCrypt);
        $nonce = random_bytes(SODIUM_CRYPTO_BOX_NONCEBYTES);
			//wp_die(var_dump(($json)));


        $encryptedText = sodium_crypto_box($dataToCrypt, $nonce, base64_decode($secretKey));
		
        $dataCrypted = base64_encode($encryptedText);
        $nonce = base64_encode($nonce);
        
    } catch (Exception $e) {
        edd_set_error('utopiapay_error', __('Error while starting the transaction: '.$e->getMessage(), 'utopiapay'));
        edd_send_back_to_checkout('?payment-mode=utopiapay');
    }

    // redirect to utopiapay gateway
    $data = base64_encode( json_encode([
        "id"=>$id,
        "data"=>$dataCrypted,
        "nonce"=>$nonce,
    ]));

    $url = $uUrl . "/checkout/pre/wp/" . $data;
    // add processing status
    edd_update_payment_status($order_id, 'pending');
    
    // redirect to utopiapay gateway
    wp_redirect($url);
}
add_action('edd_gateway_UtopiaPay', 'pw_edd_process_payment');



// listen for the IPN
function process_ipn() {

    global $edd_options;
    
    if (!empty($_POST)) {
        try{
            $posted = $_POST;

            if(isset($posted['status']) || isset($posted['data'])){
                
            }else{
                return;
            }
            

            $order = false;
            $error_msg = "Unknown error";
            $auth_ok = false;
            
                
            if (isset($posted['status']) && $posted['status'] == "cancel") {

                // updsate status
                // wp_die(var_dump($posted));
                edd_update_payment_status($posted['orderID'], 'abandoned');
                exit;
            }
    
            if (isset($posted['status']) && $posted['status'] == "paying") {

                // updsate status
                edd_update_payment_status($posted['orderID'], 'processing');
                exit;
            }
            
            if(isset($posted['status']) && $posted['status'] == "illegal"){

                // updsate status
                edd_update_payment_status($posted['orderID'], 'failed');
                exit;
            }
    

            // get data patymentID, name and status
            $data = $posted['data'];
            $data = base64_decode($data);
            $cipher = "AES-128-CTR";
            $iv = base64_decode($posted['iv']);

            // the token is the key
            $key = edd_get_option('utopiapay_token');
            $decrypted = openssl_decrypt($data, $cipher, $key, 0, $iv);
            $sadas = $decrypted;
            $decrypted = json_decode($decrypted);


            // get the derypted data from the gateway
            $paymentID = $decrypted->paymentID;
            $name = $decrypted->name;
            $status = $decrypted->status;
            $order_id = $decrypted->orderID;
            $token = $decrypted->token;
            $amount = $decrypted->amount;
            $currency = $decrypted->currency;



            // check if all is set
            if (isset($paymentID) && isset($name) && isset($status)) {
                // check if payment is successful

                edd_insert_payment_note($order_id, 'Status: ' . $status);

                if ($status == "success") {

                    // payment correct
                    edd_update_payment_status($order_id, "complete");
                    // Add note about how much user payed and the currency
                    edd_insert_payment_note($order_id, 'Amount: ' . $amount . ' ' . $currency. ' - Payment ID: ' . $paymentID);
                    edd_empty_cart();
    
                    exit;
                } else {
                    edd_update_payment_status($order_id, "failed");
                    exit;
                }
            } else if (isset($status)) {
                if ($status == "cancel") {
                    edd_update_payment_status($order_id, "abandonded");
                    edd_empty_cart();
                    exit;
                } else {
                    edd_update_payment_status($order_id, "failed");
                    exit;
                }
            } else {
                edd_update_payment_status($order_id, "failed");
                exit;
            }
        }catch (\Exception $e){
            return;
        }
    } else {
        return;
    }
}

// edd add ipn response
add_action('init', 'process_ipn');